#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look using G'MIC Linify

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-03-31: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
wrkWH=3000

############################################################################
# Service routines
#---------------------------------------------------------------------------

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

############
# kris-color
# ---------- 
def kris_color_edge (inImage, inLayer, inColor, inSize):
    
    pdb.gimp_selection_all(inImage)
    pdb.gimp_selection_shrink(inImage, inSize)
    pdb.gimp_selection_invert(inImage)
    pdb.gimp_context_set_background(inColor)
    pdb.gimp_edit_fill(inLayer, FILL_BACKGROUND)
    pdb.gimp_selection_none(inImage)
    
    return (inLayer)

####################
#   M A I N
#-------------------
    
def DrawStyleV (inImage, inDrawable, outimgWH,
                doFrame, frameCol, doFlatten) :
    
    pdb.gimp_context_push()
    # source image characteristics
    width = inImage.width
    height = inImage.height
    OrigWH = width+height
    otype = inImage.base_type
    # image name
    iname = inImage.name
    split = iname.split("(", 1)
    iname = split[0]
    end = len(iname)-2
    iname = iname[1:end]    # remove []
    #info_msg (iname)
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # working sizes
    Ratio = wrkWH*1.00/OrigWH*1.00
    rWidth = int(width*Ratio)
    rHeight = int(height*Ratio)
    rWH = rWidth+rHeight
    
    # START    
    # create output image
    newImage = pdb.gimp_image_new (rWidth, rHeight, otype )
    newBG = pdb.gimp_layer_new_from_drawable (inDrawable, newImage)
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale (newBG, rWidth, rHeight, False)
    newBG.name = inDrawable.name
    if '.' in newBG.name:
        split = newBG.name.split(".", 1)
        newBG.name = split[0]
    pdb.gimp_image_set_filename (newImage,iname)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    # adjust mean value #------------------------------------------------------------------
    (meanV,B,C,D,E,F)=pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                      0.0, 1.0)
    #info_msg ("meanV="+str(meanV))
    if meanV<120.0:  # dark images
        while meanV<120.0:  # adjust dark images
            pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE,
                                            6, (0.0,0.0, meanV/255,(meanV+8)/255, 1.0,1.0))
            (meanV,B,C,D,E,F)=pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                      0.0, 1.0)
    else:
        while meanV>135.0:  # adjust light images
            pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, meanV/255,(meanV-8)/255, 1.0,1.0))
            (meanV,B,C,D,E,F)=pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                      0.0, 1.0)
    #info_msg ("meanV adj="+str(meanV)) #---------------------------------------------------
    
    
# ==STEP1=================================================================================
       
    embossed_layer = newBG.copy()
    newImage.add_layer (embossed_layer, 0)
    pdb.plug_in_sel_gauss (newImage, embossed_layer, rWH*1.0/600, 60)
    pdb.plug_in_gauss (newImage, embossed_layer,
                       rWidth*1.0/600, rHeight*1.0/600, 0)
    pdb.plug_in_gmic_qt(newImage, newBG, 1,0 ,
    #[G'MIC] Emboss: gcd_emboss 128,0
    "-v - -gcd_emboss 191,0 ")
    embossed_layer = newImage.layers[0]
    embossed_layer.name = "black embossed"
    embossed_layer.mode = LAYER_MODE_GRAIN_MERGE
    embossed_layer.opacity = 100.0
    
    pdb.plug_in_gmic_qt(newImage, newBG, 1,0 ,
    #[G'MIC] Smooth [Anisotropic]: fx_smooth_anisotropic 250,0,0.50,0.6,1.1,0.8,30,2,0,1,1,0,0,50,50
    "-v - -fx_smooth_anisotropic 250,0,0.50,0.6,1.1,0.8,30,2,0,1,1,0,0,50,50 ")
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    Step1Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage,
                                                "step 1 Outcome")
    newImage.add_layer(Step1Outcome,0)
    Step1Outcome.mode=LAYER_MODE_NORMAL
    Step1Outcome.opacity = 100.0
    
# ==STEP2=================================================================================

    pdb.plug_in_gmic_qt(newImage, newBG, 1,2 ,
    #[G'MIC] Sharp Abstract: fx_sharp_abstract 4,10,0.5,0,0,50,50
    "-v - -fx_sharp_abstract 4,10,0.5,0,0,50,50 ")
    sharp_abstract = newImage.layers[0]
    sharp_abstract.name = "sharp abstract"
    sharp_abstract.mode = LAYER_MODE_NORMAL
    sharp_abstract.opacity = 100.0
    #pdb.plug_in_sel_gauss (newImage, sharp_abstract, rWH/600, 60)
    
# ==STEP3=================================================================================

    edges = sharp_abstract.copy()
    newImage.add_layer (edges, 0)
    edges.name = "edges"
    edges.mode = LAYER_MODE_SCREEN
    edges.opacity = 100.0
    pdb.plug_in_edge (newImage, edges, 1.0, 2, 0)   # smear, sobel
    
# ==STEP4=================================================================================

    lines = newBG.copy()
    newImage.add_layer (lines, 0)
    lines.name = "lines"
    lines.mode = LAYER_MODE_GRAIN_MERGE
    lines.opacity = 100.0
    info_msg("applying GMIC linify, please wait....")
    pdb.plug_in_gmic_qt(newImage, lines, 1,0 ,
    #[G'MIC] Linify: fx_linify 50,10,40,10,24,0,0,0,50,50
    "-v - -fx_linify 50,10,40,10,24,0,0,0,50,50 ")
    
# ==FINAL STEP=================================================================================
    
    BGonTop = newBG.copy()
    newImage.add_layer(BGonTop,0)
    BGonTop.name = "BG on Top for touch up "
    BGonTop.mode = LAYER_MODE_NORMAL
    BGonTop.opacity = 50.0
    graymask = pdb.gimp_layer_create_mask (BGonTop, ADD_MASK_COPY)
    pdb.gimp_layer_add_mask (BGonTop, graymask)
    
    Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage, "Outcome")
    newImage.add_layer(Outcome,0)
    
    if doFlatten==True:
        pdb.gimp_item_set_visible (newBG, False) # temp
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
        pdb.gimp_item_set_visible (newBG, True) # reset
        
    strokeMode = "by G'mic Emboss, Sharp abstract and Linify"
    Outcome.name = st+" "+iname+" stroked "+strokeMode
     
    if doFrame == True:
        # frame
        # - variables used
        (MeanR, B,C,D,E,F) = pdb.gimp_drawable_histogram (Outcome, HISTOGRAM_RED, 0., 1.)
        (MeanG, B,C,D,E,F) = pdb.gimp_drawable_histogram (Outcome, HISTOGRAM_GREEN, 0., 1.)
        (MeanB, B,C,D,E,F) = pdb.gimp_drawable_histogram (Outcome, HISTOGRAM_BLUE, 0., 1.)
        MeanR=max(MeanR,32)
        MeanG=max(MeanG,32)
        MeanB=max(MeanB,32)
        MeanV = (MeanR+MeanG+MeanB)/3
        if MeanV < 100:
            mult = 2.5
        elif MeanV < 150:
            mult = 1.66
        elif MeanV < 200:
            mult = 1.25
        else:
            mult = 1.00
        # define the frame colour always based on the image colours    
        if frameCol==0: # RED
            frameColor = (min(255,int(MeanR*mult)),
                          int( MeanG/1.2),
                          int( MeanB/1.2),
                          255)  # WARNING: int() is important !
        if frameCol==1: # GREEN
            frameColor = (int( MeanR/1.2),
                          min(255,int(MeanG*mult)),
                          int( MeanB/1.2),
                          255)  # WARNING: int() is important !
        if frameCol==2: # BLUE
            frameColor = (int( MeanR/1.2),
                          int( MeanG/1.2),
                          min(255,int(MeanB*mult)),
                          255)  # WARNING: int() is important !
        if frameCol==3: # YELLOW
            frameColor = (min(255,int(MeanR*mult)),
                          min(255,int(MeanG*mult)),
                          int( MeanB/1.2),
                          255)  # WARNING: int() is important !
        if frameCol==4: # CYAN
            frameColor = (int( MeanR/1.2),
                          min(255,int(MeanG*mult)),
                          min(255,int(MeanB*mult)),
                          255)  # WARNING: int() is important !
        if frameCol==5: # MAGENTA
            frameColor = (min(255,int(MeanR*mult)),
                          int( MeanG/1.2),
                          min(255,int(MeanB*mult)),
                          255)  # WARNING: int() is important !
        if frameCol==6: # AVERAGE
            frameColor = (min(255,int(MeanR*mult)),
                          min(255,int(MeanG*mult)),
                          min(255,int(MeanB*mult)),
                          255)  # WARNING: int() is important !
        if frameCol==7: # WHITE
            frameColor = (min(255,int(MeanR*3)),
                          min(255,int(MeanG*3)),
                          min(255,int(MeanB*3)),
                          255)  # WARNING: int() is important !
        #gimp.message(str(frameColor))
        frameSize = int(round(wrkWH/75))
        frameGranu = int(round(wrkWH/200))
        fWidth = int(round(rWidth+frameSize*1.0))
        fHeight = int(round(rHeight+frameSize*1.0))
        pdb.gimp_image_resize (newImage, fWidth, fHeight,
                               int(round(frameSize*0.5)), int(round(frameSize*0.5)))
        FrameLayer = pdb.gimp_layer_new(newImage, fWidth, fHeight, RGBA_IMAGE,
                        "White Border Layer", 100.0, LAYER_MODE_NORMAL)
        newImage.add_layer(FrameLayer, 0)
        pdb.gimp_edit_clear(FrameLayer)
        kris_color_edge(newImage, FrameLayer, frameColor, frameSize)
        pdb.gimp_layer_scale (FrameLayer, int(round(fWidth/frameGranu)),
                              int(round(fHeight/frameGranu)), True)
        pdb.plug_in_spread(newImage, FrameLayer, int(round(frameSize/frameGranu)),
                           int(round(frameSize/frameGranu)))
        kris_color_edge(newImage, FrameLayer, frameColor, 1)
        pdb.gimp_layer_scale(FrameLayer, fWidth, fHeight, True)
        pdb.gimp_image_select_item(newImage, CHANNEL_OP_REPLACE, FrameLayer)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_clear(FrameLayer)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_clear(FrameLayer)
        pdb.gimp_context_set_background(frameColor)
        pdb.gimp_edit_fill(FrameLayer, FILL_BACKGROUND)
        pdb.gimp_selection_none(newImage)
        kris_color_edge(newImage, FrameLayer, frameColor, 1)
        pdb.plug_in_gauss_rle(newImage, FrameLayer, frameSize, True, True)
        Outcome = pdb.gimp_image_merge_down (newImage, FrameLayer, EXPAND_AS_NECESSARY)
    else:
        pdb.gimp_item_set_visible (Outcome, False)
     
    if outimgWH==0: outimgWH=OrigWH
    if outimgWH<>wrkWH:
        uRatio = outimgWH*1.00/wrkWH*1.00
        uWidth = int(rWidth*uRatio)
        uHeight = int(rHeight*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
        
    # tier down
    
    pdb.gimp_selection_none(newImage)
    pdb.gimp_context_pop()
    
    pdb.gimp_display_new(newImage)
     
    return 

# This is the plugin registration function

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2021"
desc="Draw Effect Style V "

register(
    "DrawStyleV",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)",+
                              3000, (0, 8000, 100)),
      (PF_TOGGLE, "frame", "Add a frame?", True),
      (PF_OPTION, "framecol", "Colour of the frame"+
                    "\n  (if Frame=Yes)", 7, ["red-ish","green-ish","blue-ish",
                                            "yellow-ish","cyan-ish","magenta-ish",
                                            "image average","white-ish",
                                           ]),
      (PF_TOGGLE, "flatten", "Flatten image?", True),
    ], 
    [
      (PF_DRAWABLE, "drawable", "Output drawable", None),
    ],
    DrawStyleV,
    menu=menu
    )

main()
